/*
 * wifi.h
 *
 *  Created on: 19 Feb 2018
 *      Author: ctorr
 */

#ifndef WIFI_H_
#define WIFI_H_

#include <multos.h>

/* This version supports the following setup of the ESP8266
 *
 * See https://www.espressif.com/sites/default/files/documentation/4A-ESP8266_AT_Instruction_Set__EN.pdf
 *
AT version:1.6.0.0(Feb  3 2018 12:00:06)
SDK version:2.2.1(f42c330)
compile time:Feb 12 2018 16:31:26
Bin version(Wroom 02):1.6.1

The default baud rate needs to be changed to 57600 using the command

AT+UART_DEF=57600,8,1,0,1

The default mode needs to be set
AT+CWMODE_DEF=1
*/

// Error codes
#define WIFI_ERR_CONNECT		0x6F00	// Failed to connect to host
#define WIFI_ERR_DEVICE			0x6F01	// Module fails to respond OK to an AT
#define WIFI_ERR_OPEN			0x6F03  // AT+CIPSTART failed to respond OK
#define WIFI_ERR_SEND			0x6F04	// AT+CIPSEND didn't respond with a >
#define WIFI_ERR_TRANSMIT		0x6F05	// Data transmission failed
#define WIFI_ERR_CLOSE			0x6F06	// Connection did not close when expected


// Data sizes
#define WIFI_SERIAL_IF 1
#define WIFI_I2C_IF 2
#define WIFI_SPI_IF 3
#define WIFI_MAX_REPLY	1999
#define WIFI_MAX_SSID	63
#define WIFI_MAX_PWD 	63
#define WIFI_MAX_PROTOCOL 3
#define WIFI_MAX_SERVER 63
#define MQTT_MAX_ID		63
#define MQTT_MAX_TOPIC	49
#define MQTT_MAX_PAYLOAD 1948

// APDU commands
#define WIFI_CLA			0x70
#define WIFI_CONNECT_INS	0x00
#define WIFI_DISCONNECT_INS	0x01
#define WIFI_HTTPGET_INS	0x02
#define WIFI_RESET_INS		0x03
#define WIFI_SETUP_INS		0x04 // P1 = Interface type, P2 = Interface ID
#define WIFI_MQTT_INS		0x05 // P1 = Message type

// MQTT Messages
#define MQTT_CONNECT		0x10
#define MQTT_DISCONNECT		0xE0
#define MQTT_PUBLISH		0x30

// Public data-structures
typedef struct
{
	char acSSID[WIFI_MAX_SSID+1];
	char acPWD[WIFI_MAX_PWD+1];
} wifiConnectApdu_t;

typedef struct
{
	char acUri[WIFI_MAX_REPLY+1];   // Null terminated
	char acProtocol[WIFI_MAX_PROTOCOL+1]; // Null terminated protocol name
	WORD wPort;
	char acServer[WIFI_MAX_SERVER+1]; // Null terminated server name
	WORD wMaxWait; // Milliseconds to wait for the reply
} wifiHttpGetReqApdu_t;

typedef struct
{
	char acProtocol[WIFI_MAX_PROTOCOL+1]; // Null terminated protocol name
	WORD wPort;
	char acServer[WIFI_MAX_SERVER+1]; // Null terminated server name
	WORD wMaxWait; // Maximum time to wait in ms
	char acConnectionId[MQTT_MAX_ID+1];
} mqttConnectApdu_t;

typedef struct
{
	char acTopic[MQTT_MAX_TOPIC+1];
	WORD wPayloadLen;
	BYTE abPayload[MQTT_MAX_PAYLOAD];
} mqttPublishApdu_t;

// FUNCTIONS ----------------------------------------------------------------------------------

/// bInterfaceType - WIFI_SERIAL_IF, WIFI_I2C_IF or WIFI_SPI_IF
/// bInterfaceId - Serial port number, I2C address or SPI Slave number
WORD wifiInit(BYTE bInterfaceType, BYTE bInterfaceId);

// The connection is defined in a wifiConnectApdu_t structure in the melpublic union
WORD wifiBegin(void);

// The request is defined by a wifiHttpGetReqApdu_t structure in the melpublic union. Content as follows
// Protocol: "SSL", "TCP" or "UDP"
// Server: name of the server e.g. www.google.com
// Port: Usually 80 or 8080 for TCP and 443 for SSL
// Uri: The page on the server to retrieve. e.g. /home.html
// MaxWait: The number of milliseconds to wait before giving up
WORD wifiHttpGet(void);

// The request is defined by a mqttConnectApdu_t structure in the melpublic union. Content as follows
// Protocol: "SSL" or "TCP"
// Server: name or ip address of the MQTT server
// Port: Usually 1883
// MaxWait: The number of milliseconds to wait before giving up
// ConnectionId: Id used by MQTT to identify the source and route messages
WORD mqttConnect(void);

WORD mqttDisconnect(void);

// Publish a message in "fire and forget" mode
// The request is defined by a mqttPublishApdu_t structure in the melpublic union. Content as follows
// Topic: The topic to publish to
// Payload: The data to publish
WORD mqttPublish(void);

#endif /* WIFI_H_ */
